﻿// These units need to match up with the options in Settings.html
var DistanceUnit =
{
    Kilometres: 'km',
    StatueMiles: 'ms',
    NauticalMiles: 'mn'
};

// These units need to match up with the options in Settings.html
var HeightUnit =
{
    Feet: 'ft',
    Metres: 'mt'
};

// These units need to match up with the options in Settings.html
var SpeedUnit =
{
    Knots: 'kts',
    MilesPerHour: 'mph',
    KilometresPerHour: 'kph'
};

// An object that holds the configuration settings stored against the gadget
function Configuration()
{
    this.virtualRadarServer = 'http://127.0.0.1/VirtualRadar';
    this.latitude = 51.4;
    this.longitude = -0.6;
    this.userName = '';
    this.password = '';
    this.playSounds = false;
    this.distanceUnit = DistanceUnit.StatueMiles;
    this.heightUnit = HeightUnit.Feet;
    this.speedUnit = SpeedUnit.Knots;
    this.showFlightLevel = false;
    this.flightLevelTransition = 6000;
    this.showVerticalSpeedPerSecond = false;
};

// An object that can read and write configuration settings
function ConfigurationStorage()
{
    this.load = function ()
    {
        var result = new Configuration();
        var settingsVersion = System.Gadget.Settings.readString('settingsVersion');
        if (settingsVersion === '') settingsVersion = 0;
        else settingsVersion = Number(settingsVersion);
        if (settingsVersion >= 1) {
            result.virtualRadarServer = System.Gadget.Settings.readString('settingsVirtualRadarServer');
            result.latitude = Number(System.Gadget.Settings.readString('settingsLatitude'));
            result.longitude = Number(System.Gadget.Settings.readString('settingsLongitude'));
        }
        if (settingsVersion >= 2) {
            result.userName = System.Gadget.Settings.readString('settingsUserName');
            result.password = System.Gadget.Settings.readString('settingsPassword');
        }
        if (settingsVersion >= 3) {
            result.playSounds = System.Gadget.Settings.readString('settingsPlaySounds') === 'true';
        }
        if (settingsVersion >= 4) {
            result.distanceUnit = System.Gadget.Settings.readString('settingsDistanceUnit');
            result.heightUnit = System.Gadget.Settings.readString('settingsHeightUnit');
            result.speedUnit = System.Gadget.Settings.readString('settingsSpeedUnit');
            result.showFlightLevel = System.Gadget.Settings.readString('settingsShowFlightLevel') === 'true';
            result.flightLevelTransition = Number(System.Gadget.Settings.readString('settingsFlightLevelTransition'));
            result.showVerticalSpeedPerSecond = System.Gadget.Settings.readString('settingsShowVerticalSpeedPerSecond') === 'true';
        }

        return result;
    };

    this.save = function(configuration)
    {
        System.Gadget.Settings.writeString('settingsVersion', '4');
        System.Gadget.Settings.writeString('settingsVirtualRadarServer', configuration.virtualRadarServer);
        System.Gadget.Settings.writeString('settingsLatitude', configuration.latitude);
        System.Gadget.Settings.writeString('settingsLongitude', configuration.longitude);
        System.Gadget.Settings.writeString('settingsUserName', configuration.userName);
        System.Gadget.Settings.writeString('settingsPassword', configuration.password);
        System.Gadget.Settings.writeString('settingsPlaySounds', configuration.playSounds ? 'true' : 'false');
        System.Gadget.Settings.writeString('settingsDistanceUnit', configuration.distanceUnit);
        System.Gadget.Settings.writeString('settingsHeightUnit', configuration.heightUnit);
        System.Gadget.Settings.writeString('settingsSpeedUnit', configuration.speedUnit);
        System.Gadget.Settings.writeString('settingsShowFlightLevel', configuration.showFlightLevel ? 'true' : 'false');
        System.Gadget.Settings.writeString('settingsFlightLevelTransition', configuration.flightLevelTransition);
        System.Gadget.Settings.writeString('settingsShowVerticalSpeedPerSecond', configuration.showVerticalSpeedPerSecond ? 'true' : 'false');
    };
};
