﻿/// <reference path="Configuration.js" />
/// <reference path="jQuery-1.7.1.min.js" />

document.onreadystatechange = function()
{
    if(document.readyState === "complete") {
        var configStorage = new ConfigurationStorage();
        var config = configStorage.load();
        var configUI = new ConfigurationUI();

        $('#selectServer').click(function()        { configUI.displayPage($(this)); });
        $('#selectLocation').click(function()      { configUI.displayPage($(this)); });
        $('#selectSettings').click(function()      { configUI.displayPage($(this)); });

        $('#selectServer').hover(function()        { configUI.hoverSelector($(this), true); }, function() { configUI.hoverSelector($(this), false); });
        $('#selectLocation').hover(function()      { configUI.hoverSelector($(this), true); }, function() { configUI.hoverSelector($(this), false); });
        $('#selectSettings').hover(function()      { configUI.hoverSelector($(this), true); }, function() { configUI.hoverSelector($(this), false); });

        System.Gadget.onSettingsClosing = settingsClosingHandler;

        $('#textboxLatitude').change(function()     { if(!configUI.isUserMovingPin()) configUI.syncPinToForm(); });
        $('#textboxLongitude').change(function()    { if(!configUI.isUserMovingPin()) configUI.syncPinToForm(); });
        $('#map_movePinToCentre').click(function()  { configUI.movePinToMapCentre(); });

        configUI.displayPage($('#selectServer'));
        configUI.populate(config);

        function settingsClosingHandler(args)
        {
            if(args.closeAction === args.Action.commit) {
                if(!configUI.validateInput()) args.cancel = true;
                else {
                    configUI.copyInputToConfig(config);
                    configStorage.save(config);
                    args.cancel = false;
                }
            }
        };
    }
}

// An object to manage the UI for displaying and entering the configuration through the settings page
function ConfigurationUI()
{
    var that = this;
    var mMap;
    var mPin;
    var mMovingPin = false;
    var mMapShown = false;

    this.isUserMovingPin = function() { return mMovingPin; };

    this.displayPage = function(selector)
    {
        $('.selector').each(function() {
            if(selector.is($(this))) {
                $(this).addClass('checkedSelector');
                $(this).removeClass('uncheckedSelector');
                $(this).removeClass('selectorHover');
            } else {
                $(this).removeClass('checkedSelector');
                $(this).addClass('uncheckedSelector');
            }
        });

        var showPage = selector.children('.selectPage').text();

        $('.page').each(function(index, element) {
            if(element.id !== showPage) $(this).hide();
            else {
                $(this).show();
                $("input:visible:enabled:first").focus();
            }
        });
        
        if(selector.is('#selectLocation') && !mMapShown) showMap();
    };

    this.hoverSelector = function(selector, entering)
    {
        if(entering) {
            if(selector.hasClass('uncheckedSelector')) selector.addClass('selectorHover');
        } else {
            if(selector.hasClass('uncheckedSelector')) selector.removeClass('selectorHover');
        }
    };

    function showMap()
    {
        if(mMap) {
            google.maps.event.trigger(mMap, 'resize');
            that.syncPinToForm();
            mMapShown = true;
        }
    };
    
    this.populate = function(config)
    {
        textboxVirtualRadarAddress.value = config.virtualRadarServer;
        textboxLatitude.value = config.latitude;
        textboxLongitude.value = config.longitude;
        textboxUserName.value = config.userName;
        textboxPassword.value = config.password;
        selectDistanceUnit.value = config.distanceUnit;
        selectHeightUnit.value = config.heightUnit;
        selectSpeedUnit.value = config.speedUnit;
        checkboxShowFlightLevel.checked = config.showFlightLevel;
        textboxTransitionAltitude.value = config.flightLevelTransition;
//        checkboxPlaySounds.checked = config.playSounds;
        checkboxVsiPerSecond.checked = config.showVerticalSpeedPerSecond;

        try {
            mMap = new google.maps.Map(document.getElementById('map_canvas'), {
                center: new google.maps.LatLng(config.latitude, config.longitude),
                zoom: 8,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                streetViewControl: false
            });
            mPin = new google.maps.Marker({draggable: true, map: mMap, position: new google.maps.LatLng(config.latitude, config.longitude)});
            google.maps.event.addListener(mPin, 'dragend', pinDraggedHandler);
        } catch(err) {
            $('#map_canvas').innerHTML = 'There was a problem setting up Google Maps: ' + err.message;
        }
    };

    function pinDraggedHandler(mouseEvent)
    {
        mMovingPin = true;
        try {
            textboxLatitude.value = mouseEvent.latLng.lat();
            textboxLongitude.value = mouseEvent.latLng.lng();
        } finally {
            mMovingPin = false;
        }
    };

    this.movePinToMapCentre = function()
    {
        var mapCentre = mMap.getCenter();

        textboxLatitude.value = mapCentre.lat();
        textboxLongitude.value = mapCentre.lng();
        that.syncPinToForm();
    };

    this.syncPinToForm = function()
    {
        if(mPin) {
            try {
                var latLng = new google.maps.LatLng(textboxLatitude.value, textboxLongitude.value);
                mPin.setPosition(latLng);
                mMap.setCenter(latLng);
            } catch(err) { }
        }
    };

    this.validateInput = function()
    {
        return true;
    };

    this.copyInputToConfig = function(config)
    {
        config.virtualRadarServer = textboxVirtualRadarAddress.value;
        config.latitude = textboxLatitude.value;
        config.longitude = textboxLongitude.value;
        config.userName = textboxUserName.value;
        config.password = textboxPassword.value;
        config.distanceUnit = selectDistanceUnit.value;
        config.heightUnit = selectHeightUnit.value;
        config.speedUnit = selectSpeedUnit.value;
        config.showFlightLevel = checkboxShowFlightLevel.checked;
        config.flightLevelTransition = textboxTransitionAltitude.value;
//        config.playSounds = checkboxPlaySounds.checked;
        config.showVerticalSpeedPerSecond = checkboxVsiPerSecond.checked;
    };
}