﻿/// <reference path="Configuration.js" />
/// <reference path="jQuery-1.7.1.min.js" />
/// <reference path="Image.js" />
/// <reference path="Main.js" />
/// <reference path="VirtualRadar.js" />

// The object that represents the gadget as a whole.
function Gadget()
{
    var that = this;
    var mConfigStorage = new ConfigurationStorage();
    var mConfig = mConfigStorage.load();
    var mVirtualRadar = new VirtualRadar(mConfig.virtualRadarServer, mConfig.latitude, mConfig.longitude, mConfig.userName, mConfig.password);
    var mLastResponse;
    var mDockedFlagImage = new Image();
    var mDockedIconImages = [ new Image(), new Image() ];
    var mDockedFlagIsOperator = true;
    var mUndockedWarningVisible;
    var mUndockedAircraftImage = new Image();
    var mUndockedHeadingImage = new Image();
    var mUndockedBearingImage = new Image();
    var mUndockedOperatorFlagImage = new Image();
    var mUndockedSilhouetteImage = new Image();

    $('#dockedNotifyFlag').click(function() { mDockedFlagIsOperator = !mDockedFlagIsOperator; displayResponse(); });

    System.Gadget.onSettingsClosed = configChangedHandler;
    System.Gadget.onDock = dockClickedHandler;
    System.Gadget.onUndock = undockClickedHandler;

    displayCanvas();

    mVirtualRadar.addFetchListener(fetchReceivedHandler);
    mVirtualRadar.startFetching();

    function configChangedHandler(event)
    {
        if(event.closeAction === event.Action.commit) {
            mConfig = mConfigStorage.load();
            var wasEnabled = mVirtualRadar.getEnabled();
            mVirtualRadar.setEnabled(false);
            mVirtualRadar.setAddress(mConfig.virtualRadarServer);
            mVirtualRadar.setLatitude(mConfig.latitude);
            mVirtualRadar.setLongitude(mConfig.longitude);
            mVirtualRadar.setUserName(mConfig.userName);
            mVirtualRadar.setPassword(mConfig.password);
            mVirtualRadar.setEnabled(wasEnabled);
            if(wasEnabled) mVirtualRadar.startFetching();
        }
    };

    function dockClickedHandler()
    {
        displayCanvas();
        displayResponse();
    };

    function undockClickedHandler()
    {
        displayCanvas();
        displayResponse();
    };

    function fetchReceivedHandler(response)
    {
        mLastResponse = response;
        displayResponse();
    };

    function displayCanvas()
    {
        gadgetBackground.removeObjects();

        if(System.Gadget.docked) {
            $('#undockedContent').hide();
            gadgetBackground.src = 'Images/DockedBox.png';
            gadgetBackground.style.width = 0;
            gadgetBackground.style.width = 130;
            gadgetBackground.style.height = 122;
            $('body').width(130);
            $('body').height(122);
            $('#dockedContent').show();
        } else {
            $('#dockedContent').hide();
            gadgetBackground.src = 'Images/UndockedBox.png';
            gadgetBackground.style.width = 0;
            gadgetBackground.style.width = 400;
            gadgetBackground.style.height = 290;
            $('body').width(400);
            $('body').height(290);

            $('#undockedContent').show();
            showUndockedWarning(false);
        }
    }

    function displayResponse()
    {
        if(System.Gadget.docked) fillDockedDisplay();
        else                     fillUndockedDisplay();
    }

    function fillDockedDisplay()
    {
        var response = mLastResponse;

        var detailHtml = '';
        var flagImage;
        var headingImage;
        var bearingToImage;
        if(response && !response.suppressAircraft) {
            if(response.displayText)   detailHtml += response.displayText;
            else if(!response.closest) detailHtml += 'No aircraft in range';
            else {
                var aircraft = response.closest;
                detailHtml += '<b>' + aircraft.reg ? aircraft.reg : aircraft.icao + '</b>';
                if(mDockedFlagIsOperator && aircraft.type) detailHtml += ' :: ' + aircraft.type;
                else if(!mDockedFlagIsOperator && aircraft.call) detailHtml += ' :: ' + aircraft.call;
                detailHtml += '<br/>';

                if(aircraft.op)    detailHtml += ' ' + aircraft.op + '<br/>';

                if(aircraft.from) detailHtml += '<img src="Images/From.png" width="5px" height="5px">' + formatAirport(aircraft.from);
                if(aircraft.to)   detailHtml += '<img src="Images/To.png" width="5px" height="5px">' + formatAirport(aircraft.to);

                detailHtml += '<div id="dockedFooter">';
                var length = detailHtml.length;
                if(aircraft.alt)  detailHtml = formatConvertedValueForDocked(detailHtml, detailHtml.length !== length, convertAltitude(aircraft.alt));
                if(aircraft.spd)  detailHtml = formatConvertedValueForDocked(detailHtml, detailHtml.length !== length, convertSpeed(aircraft.spd));
                if(aircraft.km)   detailHtml = formatConvertedValueForDocked(detailHtml, detailHtml.length !== length, convertDistance(aircraft.km));
                detailHtml += '</div>';

                if(mDockedFlagIsOperator && aircraft.opIcao)      flagImage = new Image(mVirtualRadar.getOperatorFlagImageAddress(aircraft.opIcao), 'Operator ICAO code: ' + aircraft.opIcao, 85, 20);
                else if(!mDockedFlagIsOperator && aircraft.type)  flagImage = new Image(mVirtualRadar.getSilhouetteImageAddress(aircraft.type), 'Aircraft type ICAO code: ' + aircraft.type, 85, 20);
                if(!flagImage)                                    flagImage = new Image(mVirtualRadar.getBlankImageAddress(85, 20), '', 85, 20);

                if(aircraft.hdg !== undefined) headingImage = new Image(mVirtualRadar.getAircraftHeadingImageAddress(aircraft.hdg), 'Aircraft heading ' + aircraft.hdg + '&deg;', 16, 16);
                if(aircraft.brng !== undefined) bearingToImage = new Image(mVirtualRadar.getBearingToImageAddress(aircraft.brng), 'Bearing ' + aircraft.brng + '&deg; from here', 16, 16);
            }
        }

        var warning = extractWarningsFromResponse(response);
        var icons = [ null, null ];
        if(warning && warning.image) icons[1] = warning.image;
        else {
            icons[0] = headingImage;
            icons[1] = bearingToImage;
        }

        if(!mDockedFlagImage.Equals(flagImage)) {
            mDockedFlagImage = flagImage ? flagImage : new Image();
            $('#dockedNotifyFlag').html(mDockedFlagImage.Html());
        }

        for(var i = 0;i < mDockedIconImages.length;++i) {
            var existingIcon = mDockedIconImages[i];
            var newIcon = icons[i];
            if(!existingIcon.Equals(newIcon)) {
                if(!newIcon) newIcon = new Image();
                mDockedIconImages[i] = newIcon;
                $('#dockedNotifyIcon' + i).html(newIcon.Html());
            }
        }

        $('#dockedDetail').html(detailHtml);
    };

    function fillUndockedDisplay()
    {
        var response = mLastResponse;

        var warning = extractWarningsFromResponse(response);
        var hasWarning = warning && (warning.image || warning.text);
        if(response && !hasWarning) {
            fillUndockedAircraft(response.closest);
            if(mUndockedWarningVisible) showUndockedWarning(false);
        } else if(hasWarning) {
            var warningHtml = '<div id="undockedWarningIcon">' + warning.image.Html() + '</div>';
            warningHtml += '<div id="undockedWarningText">' + warning.text + '</div>';
            warningHtml += '</div>';

            $('#undockedWarning').html(warningHtml);
            if(!mUndockedWarningVisible) showUndockedWarning(true);
        }
    };

    function showUndockedWarning(show)
    {
        mUndockedWarningVisible = show;
        if(mUndockedWarningVisible) {
            $('#undockedWarning').show();
            $('#undockedDetail').hide();
        } else {
            $('#undockedWarning').hide();
            $('#undockedDetail').show();
        }
    };

    function fillUndockedAircraft(aircraft)
    {
        if(aircraft) {
            var pictureImage;
            if(aircraft.hasPicture) pictureImage = new Image(mVirtualRadar.getPictureAddress('BaseStation', aircraft.reg, aircraft.icao), 'Aircraft Picture', 200, 133);
            if(!mUndockedAircraftImage.Equals(pictureImage)) {
                mUndockedAircraftImage = pictureImage ? pictureImage : new Image();
                var html = mUndockedAircraftImage.Html();
                if(aircraft.hasPicture) html = '<a href="' + mVirtualRadar.getPictureAddress('Full', aircraft.reg, aircraft.icao) + '" target="picture">' + html + '</a>';
                $('#undockedDetailPicture').html(html);
            }

            var headingImage;
            if(aircraft.hdg !== undefined) headingImage = new Image(mVirtualRadar.getAircraftHeadingImageAddress(aircraft.hdg), 'Aircraft heading ' + aircraft.hdg + '&deg;', 16, 16);
            if(!mUndockedHeadingImage.Equals(headingImage)) {
                mUndockedHeadingImage = headingImage ? headingImage : new Image();
                $('#undockedHeading').html(mUndockedHeadingImage.Html());
            }

            var bearingImage;
            if(aircraft.brng !== undefined) bearingImage = new Image(mVirtualRadar.getBearingToImageAddress(aircraft.brng), 'Bearing ' + aircraft.brng + '&deg; from here', 16, 16);
            if(!mUndockedBearingImage.Equals(bearingImage)) {
                mUndockedBearingImage = bearingImage ? bearingImage : new Image();
                $('#undockedBearing').html(mUndockedBearingImage.Html());
            }

            var operatorFlagImage;
            if(aircraft.opIcao) operatorFlagImage = new Image(mVirtualRadar.getOperatorFlagImageAddress(aircraft.opIcao), 'Operator ICAO code: ' + aircraft.opIcao, 85, 20);
            if(!mUndockedOperatorFlagImage.Equals(operatorFlagImage)) {
                mUndockedOperatorFlagImage = operatorFlagImage ? operatorFlagImage : new Image();
                $('#undockedOperatorFlag').html(mUndockedOperatorFlagImage.Html());
            }

            var silhouetteImage;
            if(aircraft.type) silhouetteImage = new Image(mVirtualRadar.getSilhouetteImageAddress(aircraft.type), 'Aircraft type ICAO code: ' + aircraft.type, 85, 20);
            if(!mUndockedSilhouetteImage.Equals(silhouetteImage)) {
                mUndockedSilhouetteImage = silhouetteImage ? silhouetteImage : new Image();
                $('#undockedSilhouette').html(mUndockedSilhouetteImage.Html());
            }

            var typeAndModel = aircraft.type ? aircraft.type + ' ' : '';
            if(aircraft.model) typeAndModel += aircraft.model;

            $('#undockedDetailRegistration').text(aircraft.reg ? aircraft.reg : aircraft.icao);
            $('#undockedDetailSquawk').text(aircraft.squawk ? aircraft.squawk : '');
            $('#undockedDetailAltitude').text(formatConvertedValueForFlyout(convertAltitude(aircraft.alt)));
            $('#undockedDetailSpeed').text(formatConvertedValueForFlyout(convertSpeed(aircraft.spd)));
            $('#undockedDetailVerticalSpeed').text(formatConvertedValueForFlyout(convertVerticalSpeed(aircraft.vsi)));
            $('#undockedDetailDistance').text(formatConvertedValueForFlyout(convertDistance(aircraft.km)));
            $('#undockedDetailCallsign').text(aircraft.call ? aircraft.call : '');
            $('#undockedOperator').text(aircraft.op ? aircraft.op : '');
            $('#undockedTypeAndModel').text(typeAndModel);

            var routeHtml = '';
            routeHtml += buildUndockedRouteRow('From:', aircraft.from ? aircraft.from : '');
            if(aircraft.via && aircraft.via.length > 0) {
                for(var i = 0;i < aircraft.via.length;++i) {
                    routeHtml += buildUndockedRouteRow(i == 0 ? 'Via:' : '', aircraft.via[i]);
                }
            }
            routeHtml += buildUndockedRouteRow('To:', aircraft.to ? aircraft.to : '');
            $('#undockedDetailRouteBlock').html(routeHtml);
        }
    };

    function buildUndockedRouteRow(label, value)
    {
        var result = '';

        result += '<div class="undockedDetailRouteLabelValue">';
        result += '<div class="undockedDetailRouteLabel">' + label + '</div>';
        result += '<div class="undockedDetailRouteValue">' + value + '</div>';
        result += '</div>';

        return result;
    };

    function extractWarningsFromResponse(response)
    {
        var result;

        if(!response) result = { text: 'No response from server' };
        else if(response.warningText) result = { text: response.warningText.replace('"', '\'') };

        if(result) result.image = new Image(response && response.warningImage ? response.warningImage : 'Images/Warning.png', result.text, 16, 16);

        return result;
    };

    function convertAltitude(altitude, longUnitName)
    {
        var result = { value: '', units: '', description: 'Altitude' };

        var feet = Number(altitude);
        if(!isNaN(feet)) {
            switch(mConfig.heightUnit) {
                case HeightUnit.Feet:       result.value = feet; result.units = longUnitName ? 'feet' : 'ft'; break;
                case HeightUnit.Metres:     result.value = Math.round(feet * 0.3048); result.units = longUnitName ? 'metres' : 'm'; break;
            }

            if(mConfig.showFlightLevel && result.value >= mConfig.flightLevelTransition) {
                result.value = 'FL' + Math.max(0, Math.round(feet / 100));
                result.units = '';
            }
        }

        return result;
    };

    function convertSpeed(speed, longUnitName)
    {
        var result = { value: '', units: '', description: 'Speed' };

        var knots = Number(speed);
        if(!isNaN(knots)) {
            switch(mConfig.speedUnit) {
                case SpeedUnit.Knots:               result.value = Math.round(knots); result.units = longUnitName ? 'knots' : 'kts'; break;
                case SpeedUnit.KilometresPerHour:   result.value = Math.round(knots * 1.852); result.units = longUnitName ? 'km/hour' : 'kmh'; break;
                case SpeedUnit.MilesPerHour:        result.value = Math.round(knots * 1.15077945); result.units = 'mph'; break;
            }
        }

        return result;
    };

    function convertVerticalSpeed(vsi, longUnitName)
    {
        var result = { value: '', units: '', description: 'V. Speed' };

        var feet = Number(vsi);
        if(!isNaN(feet)) {
            if(mConfig.showVerticalSpeedPerSecond) feet /= 60;

            var units;
            switch(mConfig.heightUnit) {
                case HeightUnit.Feet:       result.value = Math.round(feet); units = longUnitName ? 'feet' : 'ft'; break;
                case HeightUnit.Metres:     result.value = Math.round(feet * 0.3048); units = longUnitName ? 'metres' : 'm'; break;
            }

            result.units = units + '/' + (mConfig.showVerticalSpeedPerSecond ? 'sec' : 'min');
        }

        return result;
    };

    function convertDistance(distance, longUnitName)
    {
        var result = { value: '', units: '', description: 'Distance' };

        var km = Number(distance);
        if(!isNaN(km)) {
            switch(mConfig.distanceUnit) {
                case DistanceUnit.Kilometres:       result.value = Math.floor(km * 100) / 100; result.units = longUnitName ? 'kilometres' : 'km'; break;
                case DistanceUnit.NauticalMiles:    result.value = Math.floor(km * 53.9956803) / 100; result.units = longUnitName ? 'nautical miles' : 'nmi'; break;
                case DistanceUnit.StatueMiles:      result.value = Math.floor(km * 62.1371192) / 100; result.units = longUnitName ? 'statute miles' : 'mi'; break;
            }
        }

        return result;
    };

    function formatAirport(airport)
    {
        var result = "";

        var chunks = airport.split(/,\s*/g);
        for(var i = 0;i < chunks.length;++i) {
            if(i == 0) result += chunks[i] + '<br/>';
            else {
                result += i == 1 ? '<img src="Images/ToFromSpacer.png" width="5px" height="5px">' : ', ';
                result += chunks[i];
            }
        }
        if(chunks.length > 1) result += '<br/>';

        return result;
    };

    function formatConvertedValueForDocked(html, addSeparator, converted)
    {
        var result = html;

        if(converted && converted.value !== '') {
            if(addSeparator) result += ' :: ';
            result += '<span title="' + converted.description + ' (' + converted.units + ')' + '">';
            result += converted.value + '</span>';
        }

        return result;
    };

    function formatConvertedValueForFlyout(converted, hideUnits)
    {
        var result = '';

        if(converted && converted.value !== '') {
            result += converted.value.toString();
            if(!hideUnits && converted.units) result += ' ' + converted.units;
        }

        return result;
    };
}