﻿/// <reference path="Configuration.js" />
/// <reference path="jQuery-1.7.1.min.js" />
/// <reference path="Main.js" />

// An object that talks to the virtual radar server.
function VirtualRadar(serverAddress, latitude, longitude, userName, password)
{
    var that = this;
    var mAddress;
    var mLatitude;
    var mLongitude;
    var mUserName;
    var mPassword;
    var mInterval = 1000;
    var mTimeout = 10000;
    var mFetchListeners = [];
    var mEnabled = true;
    var mXhr;
    var mRequestRunning;
    var mFullAddress;

    this.setAddress = function(serverAddress) { mAddress = serverAddress; if(mAddress && mAddress.substr(mAddress.length - 1, 1) !== '/') mAddress += '/'; };
    this.setLatitude = function(latitude)     { mLatitude = latitude; };
    this.setLongitude = function(longitude)   { mLongitude = longitude; };
    this.setUserName = function(userName)     { mUserName = userName; };
    this.setPassword = function(password)     { mPassword = password; };
    this.addFetchListener = function(address) { mFetchListeners.push(address); };

    this.getEnabled = function() { return mEnabled; };
    this.setEnabled = function(value)
    {
        if(value !== mEnabled) {
            mEnabled = value;
            if(!mEnabled) abortFetch();
        }
    };

    that.setAddress(serverAddress);
    that.setLatitude(latitude);
    that.setLongitude(longitude);
    that.setUserName(userName);
    that.setPassword(password);

    function getFetchAddress()
    {
        var result = mAddress;
        if(result) result += "ClosestAircraft.json?lat=" + mLatitude + "&lng=" + mLongitude;

        return result;
    };

    this.getOperatorFlagImageAddress = function(operatorCode)
    {
        return getFlagAddress(operatorCode, 'opflag.png');
    };

    this.getSilhouetteImageAddress = function(aircraftType)
    {
        return getFlagAddress(aircraftType, 'type.png');
    };

    function getFlagAddress(file, picture)
    {
        var result = mAddress;
        if(result) {
            result += 'Images/wdth-85/hght-20/';
            if(file) result += 'file-' + file + '/' + encodeURIComponent(picture);
            else     result += 'blank.png';
        }

        return result;
    };

    this.getBlankImageAddress = function(width, height)
    {
        var result = mAddress;
        if(result) result += 'Images/wdth-' + width + '/hght-' + height + '/Blank.png';

        return result;
    };

    this.getPictureAddress = function(size, registration, icao24)
    {
        var result = mAddress;
        if(result) {
            result += 'Images/File-' + encodeURIComponent(registration ? registration : '') + ' ' + encodeURIComponent(icao24);
            result += '/Size-' + size + '/Picture.png';
        }

        return result;
    };

    this.getAircraftHeadingImageAddress = function(heading)
    {
        var result = '';

        heading = Math.floor(Number(heading));
        if(!isNaN(heading) && mAddress) result = mAddress + 'Images/Rotate-' + heading + '/Heading.png';

        return result;
    };

    this.getBearingToImageAddress = function(bearing)
    {
        var result = '';

        bearing = Math.floor(Number(bearing));
        if(!isNaN(bearing) && mAddress) result = mAddress + 'Images/Rotate-' + bearing + '/Compass.png';

        return result;
    };

    this.startFetching = function()
    {
        if(mEnabled) {
            mFullAddress = getFetchAddress();
            raiseFetchEvent({ warningText: 'Connecting to server, please wait', warningImage: 'Images/Wait.gif', suppressAircraft: true });
            setInterval(sendGetInformationRequest, mInterval);  // repeatedly calling sendGetInformationRequest from fetchReceivedHandler with setTimeout leaked
        }
    };

    function abortFetch()
    {
        if(mRequestRunning) {
            if(mXhr) mXhr.abort();
            mRequestRunning = false;
        }
    };

    function sendGetInformationRequest()
    {
        if(!mRequestRunning) {
            mRequestRunning = true;

            try {
                mXhr = $.ajax({
                    datatype: 'text',  // 'json' doesn't do anything because the server responds with a MIME type of text - also, jQuery 1.7.1 JSON parser leaks
                    error: errorHandler,
                    success: successHandler,
                    timeout: mTimeout,
                    url: mFullAddress,
                    username: mUserName,
                    password: mPassword
                });
            } catch(e) {
            }
        }
    };

    function successHandler(text, textStatus)
    {
        if(mEnabled) {
            try {
                var response = null;
                response = json_parse(text); // $.parseJSON(mXhr.responseText) leaks under 1.6.4 and 1.7.1

                raiseFetchEvent(response);
            } catch(e) {
            }
        }
        
        mRequestRunning = false;
    };
    
    function errorHandler(jqXHR, textStatus)
    {
        if(mEnabled && textStatus !== "abort") {
            raiseFetchEvent({ warningText: 'Cannot reach server, status ' + jqXHR.status + ' (' + jqXHR.statusText + ')', displayText: 'Cannot connect' });
        }

        mRequestRunning = false;
    };

    function raiseFetchEvent(response)
    {
        for(var i = 0;i < mFetchListeners.length;++i) {
            try {
                mFetchListeners[i](response);
            } catch(e) {
            }
        }
    };
}